$ifndef _DS1390_INC_
$define _DS1390_INC_
'
' Maxim DS1390 RTCC procedures for PIC24 devices
'
' This source code is provided 'as is' and any express or implied warranties, including,
' but not limited to, the implied warranties of merchantability and fitness for a particular
' purpose are disclaimed.
'
' In no event shall the author, Crownhill or contributers be liable for any direct, indirect, incidental,
' special, exemplary, or consequential damages (including, but not limited to,  procurement of substitute
' goods or services; loss of use, data, or profits;  or business interruption) however caused and on any
' theory of liability, whether in contract, strict liability, or tort (including negligence or otherwise)
' arising in any way out of the use of this source code, even if advised of the possibility of such damage.
'
' Any views and conclusions contained in the source code and any documentation are those of the author's
' and should not be interpreted as representing official policies,
' either expressed or implied, of Crownhill Associates Ltd.
'
' For use with the Crownhill Proton24 compiler
' Written by L.Johnson 19-02-14
'
'-----------------------------------------------------------------------------------------------------------------------------
' Setup default pins for the SPI interface
'
$ifndef DS1390_CS
    $define DS1390_CS PORTB.4           ' Connects to the DS1390 CS pin
$endif
$ifndef DS1390_SCLK
    $define DS1390_SCLK PORTB.7         ' Connects to the DS1390 SCLK pin
$endif
$ifndef DS1390_DOUT
    $define DS1390_DOUT PORTB.6         ' Connects to the DS1390 DOUT pin
$endif
$ifndef DS1390_DIN
    $define DS1390_DIN PORTB.5          ' Connects to the DS1390 DIN pin
$endif
'
' DS1390 registers
'
    Symbol DS_cMSeconds = $00
    Symbol DS_cSeconds  = $01
    Symbol DS_cMinutes  = $02
    Symbol DS_cHours    = $03
    Symbol DS_cWeekDay  = $04
    Symbol DS_cDay      = $05
    Symbol DS_cMonth    = $06
    Symbol DS_cYear     = $07
    Symbol DS_cControl  = $0D
    Symbol DS_cStatus   = $0E
    Symbol DS_cCharge   = $0F

$ifndef False
    $define False 0
$endif
$ifndef True
    $define True 1
$endif
'-----------------------------------------------------------------------------------------------------------------------------
' Enable the DS1390 SPI interface
'
$define DS1390_Enable() Clear DS1390_CS
'-----------------------------------------------------------------------------------------------------------------------------
' Disable the DS1390 SPI interface
'
$define DS1390_Disable() Set DS1390_CS

'-----------------------------------------------------------------------------------------------------------------------------
' Convert a packed BCD (Binary Coded Decimal) value to a decimal value
' Input     : pValue holds the BCD value to convert to decimal
' Output    : Returns the decimal value
' Notes     : None
'
Proc DS1390_BCDToDec(pValue As Byte), Byte
    Result = (pValue >> 4) * 10
    Result = Result + (pValue & $0F)
EndProc

'-----------------------------------------------------------------------------------------------------------------------------
' Convert a decimal value to packed BCD (Binary Coded Decimal) value
' Input     : pValue holds the decimal value to convert to BCD
' Output    : Returns the packed BCD value
' Notes     : None
'
Proc DS1390_DecToBCD(pValue As Byte), Byte
    Result = ((pValue / 10) << 4)
    Result = Result | (pValue // 10)
EndProc

'-----------------------------------------------------------------------------------------------------------------------------
' Receive a byte using a software SPI interface.
' Input     : None
' Return    : Returns the byte read from the SPI bus
' Notes     : Mode 1 interface:
'           : Clock level is low in idle state, Setup on rising edge, Sample on falling edge
'
Proc DS1390_ReceiveByte(), WREG0.Byte0
    WREG1 = 8                                   ' \ 8-bit SPI loop
    Repeat                                      ' /
        WREG0 = WREG0 << 1                      ' Shift the next bit into MSB
        Set DS1390_SCLK                         ' Set SCLK high
        DelayUS 2
        Clear DS1390_SCLK                       ' Pull SCLK pin low
        WREG0 = WREG0 | DS1390_DOUT             ' Capture the current bit from the device's DOUT pin
        Dec WREG1                               ' \
    Until SRbits_Z = 1                          ' / Do all 8-bits
EndProc

'-----------------------------------------------------------------------------------------------------------------------------
' Send a byte using a software SPI interface.
' Input     : pByte holds the byte to write to the SPI bus
' Return    : None
' Notes     : Mode 1 interface:
'           : Set bit on rising edge
'
Proc DS1390_SendByte(pByte As WREG0.Byte0)
    WREG1 = 8                                   ' \ 8-bit SPI loop
    Repeat                                      ' /
        DS1390_DIN = WREG0.7                    ' Put the current outgoing bit to Device's DIN pin
        WREG0 = WREG0 << 1                      ' Shift the next bit into MSB
        Set DS1390_SCLK                         ' Set SCLK high
        DelayUS 2
        Clear DS1390_SCLK                       ' Pull SCLK pin low
        Dec WREG1                               ' \
    Until SRbits_Z = 1                          ' / Do all 8-bits
EndProc

'-----------------------------------------------------------------------------------------------------------------------------
' Write a byte to the DS1390
' Input     : pRegister holds the register to access
'           : pValue holds the byte to write to the register
' Output    : None
' Notes     : The write register's address always has bit-7 set
'
Proc DS1390_WriteRegister(pRegister As Byte, pValue As Byte)
    DS1390_Enable()                             ' Enable the SPI interface
    DS1390_SendByte(pRegister | $80)            ' Locate the write register(bit-7 set)
    DS1390_SendByte(pValue)                     ' Write the value to the register
    DS1390_Disable()                            ' Disable the SPI interface
EndProc

'-----------------------------------------------------------------------------------------------------------------------------
' Read a byte from the DS1390
' Input     : pRegister holds the register to access
' Output    : Returns the value from the register
' Notes     : None
'
Proc DS1390_ReadRegister(pRegister As Byte), Byte
    DS1390_Enable()                             ' Enable the SPI interface
    DS1390_SendByte(pRegister)                  ' Locate the read register
    Result = DS1390_ReceiveByte()               ' Read from the register
    DS1390_Disable()                            ' Disable the SPI interface
EndProc

'-----------------------------------------------------------------------------------------------------------------------------
' Set the time within the DS1390
' Input     : pSeconds, pMinutes, pHours
' Output    : None
' Notes     : None
'
Proc DS1390_SetTime(pHours As Byte, pMinutes As Byte, pSeconds As Byte)
    DS1390_Enable()                             ' Enable the SPI interface
    DS1390_SendByte(DS_cMSeconds | $80)         ' Select Milliseconds register write address
    DS1390_SendByte(0)                          ' Milliseconds value
    DS1390_SendByte(DS1390_DecToBCD(pSeconds))  ' Seconds value
    DS1390_SendByte(DS1390_DecToBCD(pMinutes))  ' Minutes value
    DS1390_SendByte(DS1390_DecToBCD(pHours))    ' Hours value
    DS1390_Disable()                            ' Disable the SPI interface
EndProc

'-----------------------------------------------------------------------------------------------------------------------------
' Set the date within the DS1390
' Input     : pDay, pMonth, pYear
' Output    : None
' Notes     : None
'
Proc DS1390_SetDate(pDay As Byte, pMonth As Byte, pYear As Byte)
    DS1390_Enable()                             ' Enable the SPI interface
    DS1390_SendByte(DS_cDay | $80)              ' Select Day register write address
    DS1390_SendByte(DS1390_DecToBCD(pDay))      ' Date value
    DS1390_SendByte(DS1390_DecToBCD(pMonth))    ' Month value
    DS1390_SendByte(DS1390_DecToBCD(pYear))     ' Year value
    DS1390_Disable()                            ' Disable the SPI interface
EndProc

'-----------------------------------------------------------------------------------------------------------------------------
' Set the weekday within the DS1390
' Input     : pWeekDay holds the day of week value (1 - 7)
' Output    : None
' Notes     : None
'
Proc DS1390_SetWeekDay(pWeekDay As Byte)
    DS1390_Enable()                             ' Enable the SPI interface
    DS1390_SendByte(DS_cWeekDay | $80)          ' Select Day of the Week register write address
    DS1390_SendByte(DS1390_DecToBCD(pWeekDay))  ' Day of Week value
    DS1390_Disable()                            ' Disable the SPI interface
EndProc

'-----------------------------------------------------------------------------------------------------------------------------
' Read the time from the DS1390
' Input     : None
' Output    : Parameter pointers: pSeconds, pMinutes, and pHours
' Notes     : None
'
Proc DS1390_GetTime(ByRef pHours As Word, ByRef pMinutes As Word, ByRef pSeconds As Word)
    DS1390_Enable()                                         ' Enable the SPI interface
    DS1390_SendByte(DS_cSeconds)                            ' Seconds register read address
    Ptr8(pSeconds) = DS1390_BCDToDec(DS1390_ReceiveByte())  ' Read the seconds value
    Ptr8(pMinutes) = DS1390_BCDToDec(DS1390_ReceiveByte())  ' Read the minutes value
    Ptr8(pHours)   = DS1390_BCDToDec(DS1390_ReceiveByte())  ' Read the hours value
    DS1390_Disable()                                        ' Disable the SPI interface
EndProc

'-----------------------------------------------------------------------------------------------------------------------------
' Read the Date from the DS1390
' Input     : None
' Output    : Parameter pointers: pDay, pMonth, and pYear
' Notes     : None
'
Proc DS1390_GetDate(ByRef pDay As Word, ByRef pMonth As Word, ByRef pYear As Word)
    DS1390_Enable()                                         ' Enable the SPI interface
    DS1390_SendByte(DS_cDay)                                ' Day  register read address
    Ptr8(pDay)   = DS1390_BCDToDec(DS1390_ReceiveByte())    ' Read the date value
    Ptr8(pMonth) = DS1390_BCDToDec(DS1390_ReceiveByte())    ' Read the month value
    Ptr8(pYear)  = DS1390_BCDToDec(DS1390_ReceiveByte())    ' Read the year value
    DS1390_Disable()                                        ' Disable the SPI interface
EndProc

'-----------------------------------------------------------------------------------------------------------------------------
' Read the WeekDay from the DS1390
' Input     : None
' Output    : Returns the week day value (1 - 7)
' Notes     : None
'
Proc DS1390_GetWeekDay(), Byte
    DS1390_Enable()                                         ' Enable the SPI interface
    DS1390_SendByte(DS_cWeekDay)                            ' Day of the Week register read address
    Result = DS1390_BCDToDec(DS1390_ReceiveByte())          ' Read the Day of the Week value
    DS1390_Disable()                                        ' Disable the SPI interface
EndProc
'-----------------------------------------------------------------------------------------------------------------------------
' Check to see if the year is a leap year
' Input     : pYear holds the full year value. i.e. 2014
' Output    : Returns true if the year is a leap year
' Notes     : If a year can be divided by 100, it is not a leap year.
'           : But there is an other exception in the Gregorian calendar: if this year can be divided by 400, it is a leap year
'           : 2000: can be divided by 400: it is a leap year
'           : 2004: can be divided by 4 and not by 100: it is a leap year
'           : 2100: can be divided by 4 and also by 100 (and not by 400) : it is not a leap year
'
Proc DS1390_IsLeapYear(pYear As Word), Bit
    Result = True
    If (pYear // 4) = 0 Then
        If (pYear // 100) <> 0 Then
            Return
        EndIf
    EndIf

    If (pYear // 400) = 0 Then
        Return
    EndIf
    Result = False
EndProc

'-------------------------------------------------------------------------------------
' Return the texts following a particular date
' Input     : pDate holds the day value 1 to 31
' Output    : Returns a string holding the text following a particular date. i.e. "st", "rd", "nd" or "th"
' Notes     : None
'
Proc DS1390_GetDateName(pDate As Byte), String * 5
    Push __OSM                                  ' Required because Str$ alters the stream value
    Result = Str$(Dec, pDate)
    Select pDate
        Case 11, 12, 13
            Result = Result + "th"
        Case Else
            pDate = DS1390_DecToBCD(pDate)
            If (pDate & $0F) = 3 Then
                Result = Result + "rd"
            ElseIf (pDate & $0F) = 2 Then
                Result = Result + "nd"
            ElseIf (pDate & $0F) = 1 Then
                Result = Result + "st"
            Else
                Result = Result + "th"
            EndIf
    EndSelect
    Pop __OSM
EndProc

'-----------------------------------------------------------------------------------------------------------------------------
' Return the name of the day
' Input     : pDayOfWeek holds the day value 1 to 7 (Sunday to Saturday)
' Output    : Returns a string holding the day's name
' Notes     : None
'
Proc DS1390_GetDayName(pDayOfWeek As Byte), String * 9

    Select pDayOfWeek
        Case 1
            Result = "Sunday"
        Case 2
            Result = "Monday"
        Case 3
            Result = "Tuesday"
        Case 4
            Result = "Wednesday"
        Case 5
            Result = "Thursday"
        Case 6
            Result = "Friday"
        Case 7
            Result = "Saturday"
        Case Else
            Clear Result
    EndSelect
EndProc

'-----------------------------------------------------------------------------------------------------------------------------
' Return the name of the month
' Input     : pMonth holds the month value 1 to 12 (January to December)
' Output    : Returns a string holding the month's name
' Notes     : None
'
Proc DS1390_GetMonthName(pMonth As Byte), String * 9

    Select pMonth
        Case 1
            Result = "January"
        Case 2
            Result = "February"
        Case 3
            Result = "March"
        Case 4
            Result = "April"
        Case 5
            Result = "May"
        Case 6
            Result = "June"
        Case 7
            Result = "July"
        Case 8
            Result = "August"
        Case 9
            Result = "September"
        Case 10
            Result = "October"
        Case 11
            Result = "November"
        Case 12
            Result = "December"
        Case Else
            Clear Result
    EndSelect
EndProc

'-----------------------------------------------------------------------------------------------------------------------------
' Initialise the DS1390
' Input     : None
' Output    : None
' Notes     : Trickle Charge register values:
'           :   %10100101 No internal diode, Internal 250 Ohm resistor
'           :   %10101001 Internal diode,    Internal 250 Ohm resistor
'           :   %10100110 No internal diode, Internal 2k Ohm resistor
'           :   %10101010 Internal diode,    Internal 2k Ohm resistor
'           :   %10100111 No internal diode, Internal 4k Ohm resistor
'           :   %10101011 Internal diode,    Internal 4k Ohm resistor
'
Proc DS1390_Initialise()
    High DS1390_CS                                  ' Disable the SPI interface
    Low DS1390_SCLK                                 ' Clock idles low
    Output DS1390_DIN                               ' Set direction for outwards communication to the DS1390
    Input DS1390_DOUT                               ' Set direction for inwards communication from the DS1390

    DS1390_WriteRegister(DS_cControl, %00000010)    ' Write to the Control register (Enable Osc, No Square Wave)
    DS1390_WriteRegister(DS_cCharge, %10101010)     ' Write to the Trickle Charge register (Internal diode, Internal 2k Ohm resistor)
EndProc

'-----------------------------------------------------------------------------------------------------------------------------
_DS1390_Main:
    DS1390_Initialise()

$endif  ' _DS1390_INC_
